"{ Package: 'stx:goodies/xtreams/core/tests' }"

"{ NameSpace: Xtreams }"

TestCase subclass:#BufferTest
	instanceVariableNames:'output buffer'
	classVariableNames:''
	poolDictionaries:''
	category:'Xtreams-Core-Tests'
!


!BufferTest methodsFor:'initialize-release'!

setUp

	buffer := Buffer on: String new
! !

!BufferTest methodsFor:'tests'!

testClear

	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer clear.
	buffer write: 4 from: 'ABCD' at: 1.
	buffer read: 4 into: output at: 1.
	self assert: output = 'ABCD'.
	self assert: buffer readPosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 8
!

testInsertInTheMiddle

	| chunk |
	chunk := 'hello'.
	buffer write: chunk; write: chunk.
	buffer writePosition: chunk size.
	buffer insert: chunk.
	self assert: buffer writePosition = (chunk size * 2).
	self assert: buffer activeSize = (chunk size * 3)
!

testRead

	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 1 into: output at: 1.
	self assert: output = '1www'.
	self assert: buffer readPosition = 1.
	self assert: buffer readSize = 3.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 5.
	buffer read: 3 into: output at: 2.
	self assert: output = '1234'.
	self assert: buffer readPosition = 4.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 8
!

testReadOverflow

	output := 'wwwwww' copy.
	buffer write: 4 from: '1234' at: 1.
	self should: [buffer read: 6 into: output at: 1] raise: Incomplete.
	self assert: output = '1234ww'.
	self assert: buffer readPosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 8
!

testTrim

	output := 'wwww' copy.
	buffer write: 8 from: '12345678' at: 1.
	buffer read: 2 into: output at: 1.
	buffer writeSkip: -2.
	buffer trim.
	self assert: buffer readPosition = 0.
	self assert: buffer readSize = 4.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 0.
	buffer read: 4 into: output at: 1.
	self assert: output = '3456'.
	self assert: buffer readPosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 4
!

testWrite

	buffer := Buffer on: 'xxxx' copy.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 0.
	self assert: buffer inactiveSize = 4.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 0.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 4.
	buffer write: 1 from: '1234' at: 1.
	self assert: buffer cache = '1xxx'.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 1.
	self assert: buffer inactiveSize = 3.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 1.
	self assert: buffer readSize = 1.
	self assert: buffer writeSize = 3.
	buffer write: 3 from: '1234' at: 2.
	self assert: buffer cache = '1234'.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 4.
	self assert: buffer inactiveSize = 0.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 0
!

testWriteGrow

	self assert: buffer cacheSize = 0.
	self assert: buffer activeSize = 0.
	self assert: buffer inactiveSize = 0.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 0.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 0.
	buffer write: 4 from: '1234' at: 1.
	self assert: (buffer cache first: 4) = '1234'.
	self assert: buffer cacheSize = 8.
	self assert: buffer activeSize = 4.
	self assert: buffer inactiveSize = 4.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 4
!

testWriteObject

	buffer := Buffer on: 'xxxx' copy.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 0.
	self assert: buffer inactiveSize = 4.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 0.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 4.
	buffer put: $1.
	self assert: buffer cache = '1xxx'.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 1.
	self assert: buffer inactiveSize = 3.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 1.
	self assert: buffer readSize = 1.
	self assert: buffer writeSize = 3.
	buffer put: $2; put: $3; put: $4.
	self assert: buffer cache = '1234'.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 4.
	self assert: buffer inactiveSize = 0.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 0.
	buffer put: $5.
	self assert: (buffer cache first: 5) = '12345'.
	self assert: buffer cacheSize >= 5.
	self assert: buffer activeSize = 5.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 5.
	self assert: buffer readSize = 5.
! !

!BufferTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

