"{ Package: 'stx:goodies/xtreams/core/tests' }"

"{ NameSpace: Xtreams }"

TestCase subclass:#ElasticBufferTest
	instanceVariableNames:'output buffer'
	classVariableNames:''
	poolDictionaries:''
	category:'Xtreams-Core-Tests'
!


!ElasticBufferTest methodsFor:'initialize-release'!

setUp

	buffer := ElasticBuffer on: String new
! !

!ElasticBufferTest methodsFor:'tests'!

testClear

	buffer write: 4 from: '1234' at: 1.
	buffer clear.
	self assert: buffer cacheSize = 0.
	self assert: buffer readPosition = 0.
	self assert: buffer readSize = 0.
	self assert: buffer writePosition = 0.
	self assert: buffer writeSize = 0
!

testRead

	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 1 into: output at: 1.
	self assert: output = '1www'.
	self assert: buffer readPosition = 1.
	self assert: buffer readSize = 3.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 5.
	buffer read: 3 into: output at: 2.
	self assert: output = '1234'.
	self assert: buffer readPosition = 4.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 8
!

testReadOverflow

	output := 'wwwwww' copy.
	buffer write: 4 from: '1234' at: 1.
	self should: [buffer read: 6 into: output at: 1] raise: Incomplete.
	self assert: output = '1234ww'.
	self assert: buffer readPosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 8
!

testReadWrap

	output := 'wwwwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 3 into: output at: 1.
	buffer write: 6 from: 'ABCDEF' at: 1.
	output := 'wwwwww' copy.
	buffer read: 6 into: output at: 1.
	self assert: output = '4ABCDE'.
	self assert: buffer readPosition = 7.
	self assert: buffer readSize = 1.
	self assert: buffer writePosition = 8.
	self assert: buffer writeSize = 7
!

testTrim

	output := 'wwww' copy.
	buffer write: 3 from: '1234' at: 1.
	buffer read: 1 into: output at: 1.
	buffer writeSkip: -1.
	buffer trim.
	self assert: buffer cacheSize = 2.
	self assert: buffer cache = '23'.
	self assert: buffer readPosition = 0.
	self assert: buffer readSize = 2.
	self assert: buffer writePosition = 2.
	self assert: buffer writeSize = 0
!

testTrimInsideOut

	buffer := ElasticBuffer on: 'xxxx' copy.
	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 3 into: output at: 1.
	buffer write: 2 from: 'AB' at: 1.
	buffer writeSkip: -1.
	buffer trim.
	self assert: buffer cacheSize = 2.
	self assert: buffer cache = '4B'.
	self assert: buffer readPosition = 0.
	self assert: buffer readSize = 2.
	self assert: buffer writePosition = 2.
	self assert: buffer writeSize = 0
!

testWrite

	buffer := ElasticBuffer on: 'xxxx' copy.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 0.
	self assert: buffer inactiveSize = 4.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 0.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 4.
	buffer write: 1 from: '1234' at: 1.
	self assert: buffer cache = '1xxx'.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 1.
	self assert: buffer inactiveSize = 3.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 1.
	self assert: buffer readSize = 1.
	self assert: buffer writeSize = 3.
	buffer write: 3 from: '1234' at: 2.
	self assert: buffer cache = '1234'.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 4.
	self assert: buffer inactiveSize = 0.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 0
!

testWriteGrow

	self assert: buffer cacheSize = 0.
	self assert: buffer activeSize = 0.
	self assert: buffer inactiveSize = 0.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 0.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 0.
	buffer write: 4 from: '1234' at: 1.
	self assert: (buffer cache first: 4) = '1234'.
	self assert: buffer cacheSize = 8.
	self assert: buffer activeSize = 4.
	self assert: buffer inactiveSize = 4.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 4
!

testWriteObject

	self assert: buffer cacheSize = 0.
	self assert: buffer activeSize = 0.
	self assert: buffer inactiveSize = 0.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 0.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 0.
	buffer put: $1.
	self assert: buffer cache first = $1.
	self assert: buffer cacheSize = 2.
	self assert: buffer activeSize = 1.
	self assert: buffer inactiveSize = 1.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 1.
	self assert: buffer readSize = 1.
	self assert: buffer writeSize = 1.
	buffer put: $2; put: $3; put: $4.
	self assert: (buffer cache first: 4) = '1234'.
	self assert: buffer cacheSize = 6.
	self assert: buffer activeSize = 4.
	self assert: buffer inactiveSize = 2.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 2.
	buffer put: $5.
	self assert: (buffer cache first: 5) = '12345'.
	self assert: buffer cacheSize >= 5.
	self assert: buffer activeSize = 5.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 5.
	self assert: buffer readSize = 5.
!

testWriteWrap

	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 2 into: output at: 1.
	buffer write: 6 from: 'ABCDEF' at: 1.
	self assert: buffer cache = 'EF34ABCD'.
	self assert: buffer cacheSize = 8.
	self assert: buffer activeSize = 8.
	self assert: buffer inactiveSize = 0.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 8.
	self assert: buffer readSize = 8.
	self assert: buffer writeSize = 0
!

testWriteWrapOverflow

	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 2 into: output at: 1.
	buffer write: 5 from: 'ABCDE' at: 1.
	buffer write: 3 from: 'XYZ' at: 1.
	self assert: (buffer cache first: 4) = 'EXYZ'.
	self assert: (buffer cache last: 4) = 'ABCD'.
	self assert: buffer cacheSize = 20.
	self assert: buffer activeSize = 11.
	self assert: buffer inactiveSize = 9.
	self assert: buffer readPosition = 1.
	self assert: buffer writePosition = 11.
	self assert: buffer readSize = 10.
	self assert: buffer writeSize = 10
! !

!ElasticBufferTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

